// QALE headers

#include "MinMaxNode.h"

/*****************************************************************************
 * Definition of class: MinMaxNode
 *****************************************************************************/

MinMaxNode::MinMaxNode(
   Function theFunction,
   Node *left,
   Node *right
)
   :
   mFunction(theFunction),
   mLeft(left),
   mRight(right)
{
   // Nothing to do
}

MinMaxNode::~MinMaxNode()
{
   delete mLeft;
   delete mRight;
}

double MinMaxNode::evaluate() const
{
   double leftValue  = mLeft->evaluate();
   double rightValue = mRight->evaluate();

   return(
      mFunction == _MIN_ ?
      (leftValue < rightValue ? leftValue : rightValue) :
      (leftValue > rightValue ? leftValue : rightValue)
   );
}

ostream &MinMaxNode::print(ostream &out) const
{
   return(
      out << (mFunction == _MIN_ ? "min" : "max") <<
             "(" << *mLeft << ", " << *mRight << ")"
   );
}

Node *MinMaxNode::copy() const
{
   return(
      new MinMaxNode(mFunction, mLeft->copy(), mRight->copy())
   );
}

void MinMaxNode::accept(Visitor &v) const
{
   v.visited((MinMaxNode *)this);

   mLeft->accept(v);
   mRight->accept(v);
}
